// support code for Eeek!  XML!  Over at MacEdition
// http://macedition.com/bolts/bolts_20030610.php

// this code is in the public domain.  have fun.

#import <Foundation/Foundation.h>
#import "XMLNode.h"

// the root node of our document
XMLNode *g_root;

// mapping table for &amp;, &gt;, and friends
NSDictionary *g_defaultEntities;;


void *createStructure (CFXMLParserRef parser,
		       CFXMLNodeRef node,
		       void *info)
{
    CFXMLNodeTypeCode type;
    type = CFXMLNodeGetTypeCode (node);

    void *result = NULL;
    XMLNode *xmlnode = NULL;
    const CFXMLElementInfo *elementInfo;

    switch (type) {

    case kCFXMLNodeTypeDocument:
	NSLog (@"making a document");
	xmlnode = [[XMLNode alloc] init];
	[xmlnode setType: (NSString *)CFXMLNodeGetString(node)];
	g_root = xmlnode;
	result = xmlnode;
	NSLog (@"    document is %p", g_root);
	break;

    case kCFXMLNodeTypeElement:
	xmlnode = [[XMLNode alloc] init];
	[xmlnode setType: (NSString *)CFXMLNodeGetString(node)];

        elementInfo = (CFXMLElementInfo *)CFXMLNodeGetInfoPtr(node);

	if (!elementInfo->isEmpty) {
	    [xmlnode setAttributes: (NSDictionary*)elementInfo->attributes];
	}
	result = xmlnode;
	NSLog (@"element.  Type is %@", CFXMLNodeGetString(node));
	break;

    case kCFXMLNodeTypeAttribute:
	NSLog (@"attribute");
	break;

    case kCFXMLNodeTypeProcessingInstruction:
	NSLog (@"processing instruction");
	break;

    case kCFXMLNodeTypeComment:
	NSLog (@"comment");
	break;

    case kCFXMLNodeTypeText:
	result = (void *) CFXMLNodeGetString (node);
	// make a copy of the string so that it can be re-used
	// safely
	result = [NSString stringWithString: result];
	NSLog (@"text: %@ (%p)", result, result);
	break;

    case kCFXMLNodeTypeCDATASection:
	NSLog (@"CDATA section");
	break;
    case kCFXMLNodeTypeDocumentFragment:
	NSLog (@"document fragment");
	break;
    case kCFXMLNodeTypeEntity:
	NSLog (@"entity");
	break;

    case kCFXMLNodeTypeEntityReference:
	// supposedly kCFXMLParserReplacePhysicalEntities will
	// replace common entities, but it doesn't ("not supported").  
	// So have to do it all ourselves
	result = [g_defaultEntities objectForKey:
					(NSString *)CFXMLNodeGetString(node)];
	NSLog (@"entity reference: %@", result);
	break;

    case kCFXMLNodeTypeDocumentType:
	NSLog (@"document type");
	break;
    case kCFXMLNodeTypeWhitespace:
	NSLog (@"whitespace");
	break;
    case kCFXMLNodeTypeNotation:
	NSLog (@"notation");
	break;
    case kCFXMLNodeTypeElementTypeDeclaration:
	NSLog (@"element type declaration");
	break;
    case kCFXMLNodeTypeAttributeListDeclaration:
	NSLog (@"attribute list declaration");
	break;
    }

    return (result);
    
} // createStructure



void addChild (CFXMLParserRef parser,
	       void *parent, void *child,
	       void *info)
{
    NSLog (@"add child: %p %p", parent, child);
    [(XMLNode*)parent addChild: (XMLNode*) child];

} // addChild



void endStructure (CFXMLParserRef parser,
		   void *node, void *info)
{
    NSLog (@"end structure");

} // endStructure



CFDataRef resolveEntity (CFXMLParserRef parser,
			 CFXMLExternalID *externalID,
			 void *info)
{
    NSLog (@"resolve entity");
    return (NULL);

} // resolveEntity



Boolean handleError (CFXMLParserRef parser,
		     CFXMLParserStatusCode error,
		     void *info)
{
    NSString *description;
    description = (NSString *) CFXMLParserCopyErrorDescription (parser);

    NSLog (@"Got an error.  description: %@, line: %d,  position: %d",
	   description,
	   CFXMLParserGetLineNumber(parser),
	   CFXMLParserGetLocation(parser));

    [description release];

    return (NO);

} // handleError



void makeDefaultEntities ()
{
    g_defaultEntities = [NSDictionary dictionaryWithObjectsAndKeys:
				      @"gt", 	@">",
				      @"lt", 	@"<",
				      @"amp", 	@"&",
				      @"quot", 	@"\"",
				      @"apos", 	@"'",
				      nil];
} // makeDefaultEntities



int main (int argc, const char *argv[]) 
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    makeDefaultEntities ();

    NSString *location;
    location = @"file:///Users/markd/Development/misc/xml/highlevel/test.xml";
    location = @"http://badgertronics.com/blog/blog-xml";

    CFURLRef sourceURL;
    sourceURL = CFURLCreateWithString (NULL,
				       (CFStringRef)location,
				       NULL); // absolute url;


    CFXMLParserCallBacks callbacks = { 0, // version
				       createStructure,
				       addChild,
				       endStructure,
				       resolveEntity,
				       handleError };

    CFXMLParserRef parser;
    parser = CFXMLParserCreate (kCFAllocatorDefault,
				NULL,			  // xmldata
				sourceURL,		  // data source
				kCFXMLParserSkipWhitespace,
				kCFXMLNodeCurrentVersion, // version of nodes
				&callbacks,
				NULL); 			  // context

    if (!CFXMLParserParse(parser)) {
	NSLog (@"Parse Failed");
    } else {
	NSLog (@"g_root is %@", g_root);
    }

    // clean up any messes we made
    [g_root release];

    CFRelease (parser);
    CFRelease (sourceURL);

    [pool release];

    return (0);

} // main
